<?php

namespace QUI\ERP\CryptoCurrency;

use Codenixsv\CoinGeckoApi\CoinGeckoClient;
use Exception;
use QUI;
use QUI\ERP\Currency\Handler;

use function array_filter;
use function array_values;
use function count;
use function floatval;
use function strtolower;

/**
 * Class Import
 */
class Import
{
    /**
     * @var null|array
     */
    protected static ?array $coinList = null;

    /**
     * @var array[]
     */
    protected static array $defaultCryptos = [
        'BTC' => [
            'coingecko' => 'bitcoin',
            'title' => [
                'en' => 'Bitcoin',
                'de' => 'Bitcoin'
            ],
            'precision' => 8
        ],
        'ETH' => [
            'coingecko' => 'ethereum',
            'title' => [
                'en' => 'Ethereum',
                'de' => 'Ethereum'
            ],
            'precision' => 18
        ],
        'LTC' => [
            'coingecko' => 'litecoin',
            'title' => [
                'en' => 'Litecoin',
                'de' => 'Litecoin'
            ],
            'precision' => 8
        ],
        'BNB' => [
            'coingecko' => 'binance',
            'title' => [
                'en' => 'Binance Coin',
                'de' => 'Binance Coin'
            ],
            'precision' => 8
        ]
    ];

    /**
     * Import default cryptocurrencies from coingecko
     *
     * @throws QUI\Permissions\Exception
     * @throws Exception
     */
    public static function importCurrenciesFromCoingecko()
    {
        QUI\Permissions\Permission::checkPermission('currency.create');

        foreach (self::$defaultCryptos as $currency => $data) {
            try {
                self::updateCurrency($currency);
                continue;
            } catch (QUI\Exception $Exception) {
            }

            QUI::getDataBase()->insert(QUI\ERP\Currency\Handler::table(), [
                'currency' => $currency,
                'rate' => self::getRate($currency),
                'precision' => $data['precision']
            ]);

            QUI\Translator::addUserVar(
                'quiqqer/currency',
                'currency.' . $currency . '.text',
                [
                    'package' => 'quiqqer/currency-cryptocurrency',
                    'datatype' => 'js,php',
                    'en' => $data['title']['en'],
                    'de' => $data['title']['de']
                ]
            );

            QUI\Translator::addUserVar(
                'quiqqer/currency',
                'currency.' . $currency . '.sign',
                [
                    'package' => 'quiqqer/currency-cryptocurrency',
                    'datatype' => 'js,php',
                    'en' => $currency,
                    'de' => $currency
                ]
            );
        }

        QUI\Translator::publish('quiqqer/currency');

        EventHandler::onQuiqqerCurrencyImport();
    }

    /**
     * @param $currency
     * @throws QUI\Database\Exception
     * @throws QUI\Exception
     */
    public static function updateCurrency($currency)
    {
        // check if already exists
        $Currency = Handler::getCurrency($currency);
        $precision = 8;

        if ($Currency->getPrecision()) {
            $precision = $Currency->getPrecision();
        }

        try {
            $rate = self::getRate($currency);
        } catch (\Exception $e) {
            QUI\System\Log::addWarning($e->getMessage());
            return;
        }

        $rate = 1 / $rate; // because rate = 23000 € in 1 BTC

        if (!empty(self::$defaultCryptos[$currency]['precision'])) {
            $precision = self::$defaultCryptos[$currency]['precision'];
        }

        if ($currency === 'USD') {
            $precision = 2;
        }

        QUI::getDataBase()->update(QUI\ERP\Currency\Handler::table(), [
            'rate' => $rate,
            'precision' => $precision
        ], [
            'currency' => $currency,
        ]);
    }

    /**
     * @param $currency
     * @return float
     *
     * @throws QUI\Exception
     * @throws Exception
     */
    public static function getRate($currency): float
    {
        $Default = QUI\ERP\Currency\Handler::getDefaultCurrency();
        $code = $Default->getCode();
        $code = strtolower($code);

        $coinId = self::getCoinId($currency);

        $client = new CoinGeckoClient();
        $data = $client->simple()->getPrice($coinId, $code);

        return floatval($data[$coinId][$code]);
    }

    /**
     * @param string $code
     * @return string
     *
     * @throws QUI\Exception
     * @throws Exception
     */
    protected static function getCoinId(string $code): string
    {
        if (self::$coinList === null) {
            $client = new CoinGeckoClient();
            $supportedCoins = $client->coins()->getList();
            self::$coinList = $supportedCoins;
        } else {
            $supportedCoins = self::$coinList;
        }

        $code = strtolower($code);
        $result = array_filter($supportedCoins, function ($entry) use ($code) {
            if (strtolower($entry['id']) === $code) {
                return true;
            }

            if (strtolower($entry['symbol']) === $code) {
                return true;
            }

            if (strtolower($entry['name']) === $code) {
                return true;
            }

            return false;
        });

        if (count($result)) {
            $values = array_values($result);

            return $values[0]['id'];
        }

        throw new QUI\Exception('Code is not supported');
    }
}
